#!/usr/bin/env python
"""This file is automatically generated by generate_version_info
It uses the current working tree to determine the revision.
So don't edit it. :)
"""

version_info = {'branch_nick': u'gracie.version-0.2.10',
 'build_date': '2009-10-05 18:35:26 +1100',
 'clean': None,
 'date': '2009-10-05 18:26:56 +1100',
 'revision_id': 'ben@benfinney.id.au-20091005072656-p8z1txbrpbiktq1l',
 'revno': 155}

revisions = {}

file_revisions = {}



if __name__ == '__main__':
    print 'revision: %(revno)d' % version_info
    print 'nick: %(branch_nick)s' % version_info
    print 'revision id: %(revision_id)s' % version_info
