# -*- coding: utf-8 -*-
#
# test/__init__.py
# Part of Gracie, an OpenID provider.
#
# Copyright © 2008–2009 Ben Finney <ben+python@benfinney.id.au>
# This is free software; you may copy, modify and/or distribute this work
# under the terms of the GNU General Public License, version 2 or later.
# No warranty expressed or implied. See the file ‘LICENSE.GPL-2’ for details.

""" Unit test suite for Gracie.
    """

import scaffold


suite = scaffold.make_suite()
